// Words up timer for SchmartBoard contest
// PROJECT MP0010
//
// Uses PIC16F913 processor
//
// D. Filicicchia
// 12-31-2010
//
//Column counts for page size control
//       1         2         3         4         5         6         7         8
//345678901234567890123456789012345678901234567890123456789012345678901234567890
//
// HARDWARE DETAILS
// Uses two LCD digits to count to 99 seconds.  Both digits use a common (COM0)
// drive because these are driven in static mode.  SEG0-7 for the right digit
// (seconds), and SEG8-14 for the left digit (tens of seconds).  Both decimal 
// points are driven together as status indicators.
//  Segments are mapped starting with LCD segment "a" as the LSB, through "g" as
// bit 6, and the MSB drives the decimal point for the units digit (LCDDATA0).
// The tens digit mapping is the same for LCDDATA1, except the d.p. is not driven
// by the LCD register for the tens
//  RB5 is the relay input to the MCU, and RA2 is a test signal (toggled every
// 20 ms).  The 20 ms delays function as a debounce filter for the contact input.
//  For this particular application, minimizing power drain was not a priority since
// the unit is only used for a few hours each week.
//  Just used internal delays since count is under 99 seconds and precision is 
// not paramount.  Should use MCU timer to get better accuracy if time permits.
//
// Programmed with PICkit 2
//
//
#include <htc.h>
#include <pic16f91x.h>

#define TRUE 1
#define FALSE 0

#define _XTAL_FREQ 4000000  // for the delay routines.  Default 4 MHz int. osc.

// configure fuses
//__CONFIGURE ();
//
//OSCCAL = _READ_OSCCAL_DATA();

#define bitset(var, bitno) ((var) |= 1UL << (bitno))
#define bitclr(var, bitno) ((var) &= ~(1UL << (bitno)))

// redefine RB5 as our relay input and rename the decimal point
#define RELAY_IN			RB5
#define RELAY_INDICATOR		SEG7COM0

// if we need it:
// variable=FLASH_READ(address);

// this is a lookup array to set the segments to look like digits 0-9.
char digit[10] = {0x3F, 0x06, 0x5B, 0x4F, 0x66, 0x6D, 0x7c, 0x07, 0x7F, 0x67};

volatile char sec_count;
volatile char test_count;
volatile char tc;
volatile char count_up;


// function prototype
void display_count (char);
void blank_digits (void);


// function to convert and display digits from BCD count
void display_count (char c)
{
   char x;
   // first deal with the units digit
   if (SEG7COM0 == 1)  // decimal point is on
   { // look up digit for lower nibble and turn decimal point on
      LCDDATA0 = digit[(c & 0x0F)];
      LCDDATA0 |= 0x80;
   }
   else
   {  // look up digit and leave off the dp since it already was
      LCDDATA0 = digit[(c & 0x0F)];
   }
   
   // now do the tens digit lookup for upper nibble (tens)
   x = ((c & 0xF0) >> 4);  // isolate upper nibble value
   if (x == 0) LCDDATA1 = 0;  // blank a leading 0 at the 10's digit
   else LCDDATA1 = digit[x];

} // end of display_count function

void blank_digits (void)
{  // turn off all segments to blank display
   LCDDATA0 = 0;
   if (SEG7COM0 == 1) LCDDATA0 = 0x80;  // don't mess with the decimal point
   LCDDATA1 = 0;
}

// ***********************************************************************
void main (void)
{
 // Set some I/O ports
   TRISB4 = 0; // Make COM0 (RB4) an output
   TRISA2 = 0; // Make RA2 an output - test signal

   RB4 = 1; // set RB4 hi since the segement (inputs) are all hi at powerup
   
   //RB5 = 1; // set RB5 hi; arbitrary.
   // Use RB5 as an input; need global enable for weak pullup on this pin
   RBPU = 0;   // enables port b weak pullups

// Initialize LCD registers.  Uses recommended sequence.
   LCDPS = 0x31;   // LCD Active, Write Allowed, 1:1 prescaler
   LCDSE0 = 0xFF;   // Units digit. LCD Segment enables: SEG7:0
   LCDSE1 = 0x7F;   // Tens digit. LCD Segment enables: SEG14:8

 // Set up for LCD static drive for using COM0. Use default clock source.
   LMUX0 = 0;  // LMUX0
   LMUX1 = 0;  // LMUX1
 
// LCDCON = 0;  // needs work
   LCDDATA0 = 0x71;  // initial pixels for ones digit (F)
   LCDDATA1 = 0x5E;  // initial pixels for tens digit (d)
// Interrupts??
   LCDEN = 1;  // Enable LCD driver module
   //bitclr(LCDCON, 7);

   __delay_ms(250);  // delay 1/4 second to show initial LCD data

   test_count = 0;
   tc = 0;
   display_count(tc);

// ***********************************************************************
// main forever loop
   while (1){
      RA2 = 1;   //set the test bit hi
      __delay_ms(20);   // delay 20 ms

      if (RELAY_IN == 0)
      {
         RELAY_INDICATOR = 1;   //dp on means relay contacts are closed
         count_up = TRUE; 
      }
      else
      {
         RELAY_INDICATOR = 0;   //dp off means relay contacts are open
         count_up = FALSE; 
         test_count = 0;
         tc = 0;
      }


      RA2 = 0;   // set the test bit lo
      __delay_ms(20);	

      if (count_up == TRUE)
      {
         test_count++;
         if (test_count == 25)
         {
            test_count = 0;
            tc++; // test as a 1 second counter
            if ((tc & 0x0F) == 0x0A)
            { // count in BCD
               tc = tc + 6; 
            } 
            if (tc == 0xA0) tc = 0x99;  // saturate at 99 seconds.
            display_count(tc);
         }

      } // end if count up is true
      else 
      {
          blank_digits();  // blank the digits if we're not counting
      } 
      
   } // end of forever loop

}  // end of main
